Package: GSEABenchmarkeR
Type: Package
Title: Reproducible GSEA Benchmarking
Version: 1.33.1
Authors@R: c(
    person("Ludwig", "Geistlinger", , "ludwig.geistlinger@gmail.com", role = c("aut", "cre")),
    person("Gergely", "Csaba", role = "aut"),
    person("Mara", "Santarelli", role = "ctb"),
    person("Lucas", "Schiffer", role = "ctb"),
    person("Marcel", "Ramos", role = "ctb"),
    person("Ralf", "Zimmer", role = "aut"),
    person("Levi", "Waldron", role = "aut"),
    person("NCI", role = "fnd",
           comment = c(GrantNo. = "U24CA289073"))
  )
Description: 
    The GSEABenchmarkeR package implements an extendable framework for
    reproducible evaluation of set- and network-based methods for enrichment
    analysis of gene expression data. This includes support for the efficient
    execution of these methods on comprehensive real data compendia (microarray
    and RNA-seq) using parallel computation on standard workstations and
    institutional computer grids. Methods can then be assessed with respect to
    runtime, statistical significance, and relevance of the results for the
    phenotypes investigated.
URL: https://github.com/waldronlab/GSEABenchmarkeR
BugReports: https://github.com/waldronlab/GSEABenchmarkeR/issues
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.5.0), Biobase, SummarizedExperiment
Imports: AnnotationDbi, AnnotationHub, BiocFileCache, BiocParallel,
        edgeR, EnrichmentBrowser, ExperimentHub, grDevices, graphics,
        KEGGandMetacoreDzPathwaysGEO, KEGGdzPathwaysGEO, methods,
        S4Vectors, stats, utils
Suggests: BiocStyle, GSE62944, knitr, rappdirs, rmarkdown
biocViews: ImmunoOncology, Microarray, RNASeq, GeneExpression,
        DifferentialExpression, Pathways, GraphAndNetwork, Network,
        GeneSetEnrichment, NetworkEnrichment, Visualization,
        ReportWriting
VignetteBuilder: knitr
RoxygenNote: 7.1.0
git_url: https://git.bioconductor.org/packages/GSEABenchmarkeR
git_branch: devel
git_last_commit: 8f752f6
git_last_commit_date: 2026-04-29
Repository: Bioconductor 3.24
Date/Publication: 2026-05-03
NeedsCompilation: no
Packaged: 2026-05-03 22:05:27 UTC; biocbuild
Author: Ludwig Geistlinger [aut, cre],
  Gergely Csaba [aut],
  Mara Santarelli [ctb],
  Lucas Schiffer [ctb],
  Marcel Ramos [ctb],
  Ralf Zimmer [aut],
  Levi Waldron [aut],
  NCI [fnd] (GrantNo.: U24CA289073)
Maintainer: Ludwig Geistlinger <ludwig.geistlinger@gmail.com>
