#' @import Biobase SummarizedExperiment KEGGdzPathwaysGEO
#'         KEGGandMetacoreDzPathwaysGEO
#' @importFrom S4Vectors DataFrame
#' @importMethodsFrom S4Vectors metadata
#' @importFrom grDevices dev.off pdf rainbow
#' @importFrom graphics abline axis barplot box boxplot matlines mtext par plot
#'              points text
#' @importFrom methods is
#' @importFrom stats cor median p.adjust sd weighted.mean
#' @importFrom utils data read.delim relist setTxtProgressBar txtProgressBar 
#'              untar write.table
NULL
